function [Weight,bias] = Fast_LogisticalRegress_CG_balance(patterns,targets,lambda,options)
% The program perfroms a fast implementation of L1 regularized logistic
% regression using Direct Gradient Method (DGM) to solve the following 
% problem with two balanced classes. See reference for details.
%
% 		\min_{w,b} \sum_i  c_i ln(1+exp(-y_i*(W'X_i+b))+ lambda |W|_1
%
%       where y   = {+1, -1}
%             c_i = 1                       for y_i=1 
%             c_i = sum(y==1)/sum(y==-1)    for y_i=-1
% 
% USAGE:
%   patterns:   n*m matrix representing m data points with n dimension
%               each column represents a data point, each row
%               represents one dimension.
%
%   targets:    m-dimension class label vector = {+1, -1}
%
%   lambda:     regularization parameter
%               use lambdamax=findlambdamax_balance(patterns,targets) to
%               determine the best possible value of lambda.
% 
%   options:    option structure with the following fields with default
%               values in brackets. Set to [] to use default settings.
%               Progress is defined as difference b/w
%               two consecutive objective function values during iteration.
%
%               options.tol:        stopping criterion of iteration (1e-6)
%               options.obj:        objective cost for stopping iteration
%               options.err_style   if not present, stop when the progress 
%                                   in one iteration is below option.tol.
%                                   if present, stop when the difference b/w
%                                   average cost and option.obj is less
%                                   than option.tol
%               options.purgethres  threshold of setting a weight to 0 
%                                   (1e-6)
%               options.cg_thres    progress threshold of using conjugate 
%                                   gradient (1e-2)
%               options.sflag       if present, perform an implicit 
%                                   standardization. 
%--------------------------------------------------------------------------
% Please cite the following article if you're using this code.
% Reference:
%   Y. Cai, Y. Sun, Y. Cheng, J. Li, and S. Goodison, Fast Implementation 
%   of L1 Regularized Learning Algorithms Using Gradient Descent Methods, 
%   Proc 10th SIAM International Conference on Data Mining (SDM), 
%   pp. 862-871, April 2010.
%% ==============================================================================

if (~isfield(options,'tol'))
    options.tol=1e-6;
end

if (~isfield(options,'purgethres'))
    options.purgethres=1e-6;
end

if (~isfield(options,'cg_thres'))
    options.cg_thres=1e-2;
end

if (isfield(options,'sflag'))
    [patterns,m,mstd]=zscore(patterns, 0, 2);
    vstd=[mstd;mstd];
end
%--------------------------------------------------------------------------
% Initialization
[dim,num_samp]=size(patterns);
Weight = ones(2*dim,1);
bias = 0;
Weight = Weight/sqrt(dim*2);

sampfact=ones(num_samp,1);
sampfact(targets==-1)=sum(targets==1)/sum(targets==-1);
Original_dim = dim;
Original_index = 1:2*dim;
patterns=[patterns;-patterns];

P.a=zeros(1,num_samp);
P.lambda = lambda;
P.Targets = targets;
P.Sampfact=sampfact;

sigmoid=ones(num_samp,1)*0.5;
if (isfield(options,'sflag'))
    P.normw=Weight'*(Weight./vstd);
else
    P.normw=norm(Weight).^2;
end

cost=-sum(sampfact.*log(sigmoid))+lambda*P.normw;
progress=0;
gradient=zeros(dim*2,1);
%--------------------------------------------------------------------------
% Main loop
for j=1:10000
    if (j >1 && progress < options.cg_thres)
        gradnorm=gradient'*gradient+grad_bias*grad_bias;
    end
    
    sigmoid = 1./(1+exp(targets.*P.a'));     % Probability of being class 1
    difference = -targets.*sigmoid.*sampfact;
    gradient =(patterns*difference).*Weight;
    grad_bias= sum(difference)/2;
    
    if (isfield(options,'sflag'))
        gradient=gradient+lambda*(Weight./vstd);
    else
        gradient=gradient+lambda*Weight;
    end
    
    if (j >1 && progress < options.cg_thres)
        beta=(gradient'*gradient+grad_bias*grad_bias)/gradnorm;
        descent=gradient+beta*descent;
        descent_bias=grad_bias+beta*descent_bias;
        angle=acos((descent'*gradient+grad_bias*descent_bias)...
                   /(norm([descent;descent_bias])*norm([gradient;grad_bias])))*180/pi;
        if (angle >80)
            descent=gradient;
            descent_bias=grad_bias;
        end
    else
        descent=gradient;
        descent_bias=grad_bias;
    end
    
    Vdv=Weight.*descent;
    P.VdvX=Vdv'*patterns;
    if (isfield(options,'sflag'))
        P.normVdv=sum(Vdv./vstd);
    else
        P.normVdv=sum(Vdv);
    end
        
    dv2=descent.^2;
    P.dv2X=dv2'*patterns;

    if (isfield(options,'sflag'))
        P.normdv2=sum(dv2./vstd);
    else
        P.normdv2=sum(dv2);
    end
    P.descent_bias=descent_bias;
    
    % compute alpha via line search
    tau=0.618;
    t=1;
    for itau=1:32
        if (LR_cost_Fast_balance(t,P) < cost); break; end
        t=t*tau;
    end
    newcost= LR_cost_Fast_balance(t,P);
    
    if (newcost < cost)
        [alpha,costalpha]=MyGoldenSectionSearch(@LR_cost_Fast_balance,t, P,newcost);
        if (costalpha <newcost)
            newcost=costalpha;
        else
            alpha=t;
        end
    else
        alpha=0;
    end

    progress = (cost-newcost)/newcost;
    Weight = Weight-alpha*descent;
    bias = bias-alpha*descent_bias;
    P.a=P.a - 2*alpha*P.VdvX+alpha*alpha*P.dv2X - alpha*P.descent_bias;
    cost=newcost;
    
    if (isfield(options,'sflag'))
        P.normw=Weight'*(Weight./vstd);
    else
        P.normw=norm(Weight).^2;
    end

    %remove weights close to zero
    index_zeros = find(abs(Weight)<options.purgethres *max(abs(Weight))); 
    Weight(index_zeros)=[];
    gradient(index_zeros)=[];
    descent(index_zeros)=[];
    patterns(index_zeros,:)=[];
    
    if (isfield(options,'sflag'))
        vstd(index_zeros)=[];
    end
    Original_index(index_zeros)=[];
   
    % terminate the loop 
    if (isfield(options,'err_style'))
        if (cost/num_samp - options.obj < options.tol); break; end
    else
        if (progress < options.tol); break; end
    end
end
% end of main loop

temp = zeros(Original_dim,1);
index_pos=Original_index(Original_index<=Original_dim);
dim_pos=length(index_pos);
index_neg=Original_index(Original_index>Original_dim)-Original_dim;
dim_neg=length(index_neg);
if (dim_pos>0)
    temp(index_pos) = Weight(1:dim_pos).^2;
end
if (dim_neg>0)
    temp(index_neg) = temp(index_neg) - Weight(dim_pos+1:dim_pos+dim_neg).^2;
end
Weight = temp;

if (isfield(options,'sflag'))
    Weight=Weight./mstd;
    bias=bias-(Weight'*m);
end

%==================End of the code ========================================

function cost=LR_cost_Fast_balance(alpha, P)
% Cost function

Targets=P.Targets;
lambda=P.lambda;
    
a=P.a - 2*alpha*P.VdvX+alpha*alpha*P.dv2X - alpha*P.descent_bias;
cost=sum(P.Sampfact.*log(1+exp(-Targets.*a(:))))...
         +lambda*(P.normw-2*alpha*P.normVdv+alpha*alpha*P.normdv2);
     
%--------------------------------------------------------------------------
function [s_opt v_opt]=MyGoldenSectionSearch(LineFunc,smax,P,v_end)
s_start=0;
s_end=smax;
v_opt=v_end;
s_opt=s_end;

s1=s_start+(s_end-s_start)*0.618;
s2=s_start+(s_end-s_start)*0.382;
v1=LineFunc(s1,P);
v2=LineFunc(s2,P);

for it=1:10
    if (v1 >v_opt && v2 > v_opt)
        return
    end
    if (v1 > v2)
        v_opt=v2;
		s_opt=s2;
		v1=v2;
		s_end=s1;
		s1=s2;
		s2=s_start+(s_end-s_start)*0.382;
        v2=LineFunc(s2,P);
    else
		v_opt=v1;
		s_opt=s1;
		v2=v1;
		s_start=s2;
		s2=s1;
		s1=s_start+(s_end-s_start)*0.618;
		v1=LineFunc(s1,P);
    end
end
if (v1 < v_opt)
	v_opt=v1;
	s_opt=s1;
end
if (v2 < v_opt)
	v_opt=v2;
	s_opt=s2;
end

